##  HV.eventmerge.pl
##
##  This Perl program merges a set of event files produced by the TABARI .project files
##  listed in the file "projects.list". It processes each of the .project files to determine
##  the most recent version, then just concatenates the files based on their order in
##  "projects.list". 
##   
##  TO RUN PROGRAM:
##
##  HV.eventmerge.pl outputfile
##
##  where outputfile is the name for the merged file
##
##  PROGRAMMING NOTES:
##
##  1. This was written to combine the event files produced by multiple copies of
##     TABARI running on a parallel cluster.
##
##  SYSTEM REQUIREMENTS
##  This program has been successfully run under Red Hat Linux 3.4.6-3; it is standard perl
##  so it should also run in Windows. 
##
##  PROVENANCE:
##  Programmer: Philip A. Schrodt
##              Kansas Event Data System Project
##              Center for International Political Analysis
##              University of Kansas 
##	            Lawrence, Kansas, 66045 U.S.A.
##	            http://web.ku.edu/keds
##
##	Copyright (c) 2009  Philip A. Schrodt.  All rights reserved.
##
## 	Redistribution and use in source and binary forms, with or without modification,
## 	are permitted under the terms of the GNU General Public License:
## 	http://www.opensource.org/licenses/gpl-license.html
##
##	Report bugs to: schrodt@ku.edu
##
##	For plausible indenting of this source code, set the tab size in your editor to "2"
##
##  REVISION HISTORY:
##  24-Jun-09:  Initial version
##
##  ----------------------------------------------------------------------------------

#!/usr/local/bin/perl

$outname = $ARGV[0];

$km= 0;
open(FDIR,"projects.list")  or die "Can\'t open projects.list file; error $!";
while ($file_name = <FDIR>) {
	print $file_name;
	chomp($file_name);
	open(FIN,$file_name)  or die "Can\'t open input file; error $!";
	
	while ($line = <FIN>) {
		if ($line =~ m/<eventfile>\s*/) {               # find the event file line
			$evtname = $';                                # get the file name 
			if ($evtname =~ m/\s*$/) { $evtname = $`;}    # trim any trailing blanks
		}
		if ($line =~ m/<session/) { $session = $line;}  # gets the final session record
	}
	$session =~ m/<session (\d+)>/;  # extract the number of the final session
	$sessno = $1;
	close(FIN) or die "Can\'t close input file ; error $!";
	
	$file = "$evtname.$sessno";     # this should be the most recent version of the file
	if (-e $file) {     # see if it really exists
		++$km;
		if ($km == 1) { system("cp $file $outname");}  # copy the first file
		else {system("cat $file >> $outname");}  # append the remainder >> being the Unix 'append file' operator. But you knew that...
	}
	else { print "Can\'t find file $file\nExiting\n";} 
#	if ($km > 4) {exit;}   # debugging
}
close(FDIR) or die "Can\'t close output file ; error $!";
print "Program has finished!\n";
