##  LN.data.divide.pl
##
##  This Perl program takes the input file 'seqsort.LN.unique.txt' and copies
##  only those records meeting a hard-coded date criterion. Output is written
##  in 500,000 record blocks with a hard-coded prefix. This was used to do the final
##  chronological ordering of the data, and also to isolate records with probable
##  junk dates.
##   
##  TO RUN PROGRAM:
##
##  LN.data.divide.pl 
##
##  where filename is the name of file to be filtered. Output will be in the file
##  select.n.filename where 'n' is sequential
##
##  PROGRAMMING NOTES:
##
##  1. Detecting the string  m/^\d\d\d\d\d\d \d\d\d/  is a more robust way of 
##     getting the record header...might modify this.
##
##  SYSTEM REQUIREMENTS
##  This program has been successfully run under Mac OS 10.5; it is standard perl
##  so it should also run in Unix or Windows. 
##
##  PROVENANCE:
##  Programmer: Philip A. Schrodt
##              Kansas Event Data System Project
##              Center for International Political Analysis
##              University of Kansas 
##	            Lawrence, Kansas, 66045 U.S.A.
##	            http://web.ku.edu/keds
##
##	Copyright (c) 2009  Philip A. Schrodt.  All rights reserved.
##
## 	Redistribution and use in source and binary forms, with or without modification,
## 	are permitted under the terms of the GNU General Public License:
## 	http://www.opensource.org/licenses/gpl-license.html
##
##	Report bugs to: schrodt@ku.edu
##
##	For plausible indenting of this source code, set the tab size in your editor to "2"
##
##  REVISION HISTORY:
##  11-Jun-09:  Initial version
##
##  ----------------------------------------------------------------------------------

#!/usr/local/bin/perl

# ======== main program =========== #

#$output_prefix = ">LN.date00.06.";
#$output_prefix = ">LN.date07.09.";
$output_prefix = ">LN.date97.99.";
#$output_prefix = ">LN.date90.96.";
#$output_prefix = ">LN.date80.89.";
#$output_prefix = ">LN.date.10.79.";
$kfile = 1;

open(FIN,"seqsort.LN.unique.txt")  or die "Can\'t open input file; error $!";
open(FOUT,"$output_prefix$kfile") or die "Can\'t open output file ; error $!";

$kb=0;
while ($line = <FIN>) {
	if ($line =~ m/^(\d\d\d\d\d\d) \d\d\d/) {   # get the record header
#		if ($1 =~ m/0[0123456]\d\d\d\d/) {    # select 2000-2006 records
#		if ($1 =~ m/0[789]\d\d\d\d/) {        # select 2007-2009 records
		if ($1 =~ m/9[789]\d\d\d\d/) {        # select 1997-1999 records
#		if ($1 =~ m/9[0123456]\d\d\d\d/) {    # select 1990-1996 records
#		if ($1 =~ m/8\d\d\d\d\d/) {    # select 1980-1989 records
#		if ($1 =~ m/[1234567]\d\d\d\d\d/) {    # select 1910-1979 records, which are probably junk
			if ((++$kb%100000) == 0) {
				print $kb,"\n";   # show that the program is running
				if (($kb%500000) == 0) {
					close(FOUT) or die "Can\'t close output file ; error $!";
					++$kfile;
					open(FOUT,"$output_prefix$kfile") or die "Can\'t open output file ; error $!";
					print "New file: $output_prefix$kfile\n";
				}
			}
			$head = $line;
			$line = <FIN>;
			if ($line !~/^"/) {  # skip lines beginning with quotes
				print FOUT $head;
				print FOUT $line;
				while ($line = <FIN>) {
					print FOUT $line;
					if (length($line) == 1) {last;}
				}
			}
		}
	}
}
print "Finished reading...\n";
close(FOUT) or die "Can\'t close output file ; error $!";
close(FIN) or die "Can\'t close input file ; error $!";
print "Program has finished!\n";
