##  LN.data.filemerge.pl
##
##  This Perl program takes a series of pairs sorted TABARI-formatted input file and merges 
##  them -- it is intended to be run iteratively so to produce a completely sorted set
##  of records. $ARG[0] contains a number that distinguishes the list of sorted files, and
##  the output file
##   
##  TO RUN PROGRAM:
##
##  LN.data.filemerge.pl iteration
##
##  where iteration corresponds to a file named "sort.$ARGV[0].files" that contains a list,
##  divisible by two, of the files to be merged.  Output will be in files named 
##  "merge.$ARGV[0].sequence# 
##
##  PROGRAMMING NOTES:
##
##  1. Sequence of runs will take the form
##
##      perl LN.data.filemerge.pl 1
##      ls merge.1* > sort.2.list
##      perl LN.data.filemerge.pl 2
##      ls merge.2* > sort.3.list
##      perl LN.data.filemerge.pl 3
##
##    with occasional editing of the sort.* files to keep them aligned with pairs of
##    files.
##
##  2. It should be easy to modify the program to eliminate the "divisible by two" 
##     requirement -- just copy the final file -- and with that, one could do the 
##     binary iteration in a loop, removing the need to run the program more than
##     once. This additional code is left as an exercise.
##
##  SYSTEM REQUIREMENTS
##  This program has been successfully run under Mac OS 10.5; it is standard perl
##  so it should also run in Unix or Windows. 
##
##  PROVENANCE:
##  Programmer: Philip A. Schrodt
##              Kansas Event Data System Project
##              Center for International Political Analysis
##              University of Kansas 
##	            Lawrence, Kansas, 66045 U.S.A.
##	            http://web.ku.edu/keds
##
##	Copyright (c) 2009  Philip A. Schrodt.  All rights reserved.
##
## 	Redistribution and use in source and binary forms, with or without modification,
## 	are permitted under the terms of the GNU General Public License:
## 	http://www.opensource.org/licenses/gpl-license.html
##
##	Report bugs to: schrodt@ku.edu
##
##	For plausible indenting of this source code, set the tab size in your editor to "2"
##
##  REVISION HISTORY:
##  16-Jun-09:  Initial version
##
##  ----------------------------------------------------------------------------------

#!/usr/local/bin/perl

sub read1
{
while ($line = <FIN1>) {
		if ($line =~ m/^\d\d\d\d\d\d /) {   # get the date and record number
			$r1 = $line;
			while ($line = <FIN1>) {
				if (length($line) == 1) {return;}
				$r1 = $r1.$line;
			}
		}
	}
}

sub read2
{
	while ($line = <FIN2>) {
		if ($line =~ m/^\d\d\d\d\d\d /) {   # get the date and record number
			$r2 = $line;
			while ($line = <FIN2>) {
				if (length($line) == 1) {return;}
				$r2 = $r2.$line;
			}
		}
	}
} # sub read2

# ======== main program =========== #

$iter = $ARGV[0];
print "File iteration: $iter\n";

$km= 0;
#open(FDIR,"sort.list")  or die "Can\'t open sort.list file; error $!";
open(FDIR,"sort.$iter.list")  or die "Can\'t open sort.list file; error $!";
while ($file_name1 = <FDIR>) {
	$file_name2 = <FDIR>;
	open(FIN1,$file_name1)  or die "Can\'t open input file; error $!";
	open(FIN2,$file_name2)  or die "Can\'t open input file; error $!";
	print $file_name1,$file_name2;

	++$km;
	$output_name = ">merge.$iter.".$km;
	open(FOUT,$output_name) or die "Can\'t open output file ; error $!";
	
	$ka = 0;
	
	&read1;
	&read2;
	
	while (!eof(FIN1) && !eof(FIN2)) {
		if ($r1 le $r2) {
			print FOUT $r1,"\n";
			&read1;
		}
		else {
			print FOUT $r2, "\n";
			&read2;
		}
	}
	
	if (!eof(FIN1)) {
		while ($line = <FIN1>) {print FOUT $line;}
	}
	
	if (!eof(FIN2)) {
		while ($line = <FIN2>) {print FOUT $line;}
	}
	
	close(FOUT) or die "Can\'t close output file ; error $!";
	close(FIN1) or die "Can\'t close input file ; error $!";
	close(FIN2) or die "Can\'t close input file ; error $!";
#	if ($km>=2) {last;}
}
print "Program has finished!\n";
