##  LN.data.select.pl
##
##  This Perl program takes a TABARI-formatted input file with LN record headers and
##  selects only those records with sentence numbers 001-004 and where the first
##  character in the story is not \" or \'. Also breaks the file into 500,000 story blocks.
##   
##  TO RUN PROGRAM:
##
##  perl  LN.data.select.pl filename
##
##  where filename is the name of file to be filtered. Output will be in the file
##  select.n.filename where 'n' is sequential
##
##  PROGRAMMING NOTES:
##
##  1. Detecting the string  m/^\d\d\d\d\d\d \d\d\d/  is a more robust way of 
##     getting the record header...might modify this.
##
##  SYSTEM REQUIREMENTS
##  This program has been successfully run under Mac OS 10.5; it is standard perl
##  so it should also run in Unix or Windows. 
##
##  PROVENANCE:
##  Programmer: Philip A. Schrodt
##              Kansas Event Data System Project
##              Center for International Political Analysis
##              University of Kansas 
##	            Lawrence, Kansas, 66045 U.S.A.
##	            http://web.ku.edu/keds
##
##	Copyright (c) 2009  Philip A. Schrodt.  All rights reserved.
##
## 	Redistribution and use in source and binary forms, with or without modification,
## 	are permitted under the terms of the GNU General Public License:
## 	http://www.opensource.org/licenses/gpl-license.html
##
##	Report bugs to: schrodt@ku.edu
##
##	For plausible indenting of this source code, set the tab size in your editor to "2"
##
##  REVISION HISTORY:
##  11-Jun-09:  Initial version
##
##  ----------------------------------------------------------------------------------

#!/usr/local/bin/perl

# ======== main program =========== #

if (length($ARGV[0]) < 4) {
  print "File name is required to run the program\n";
  exit;
}

$kfile = 1;
$output_name = ">select.$kfile.".$ARGV[0];
open(FIN,$ARGV[0])  or die "Can\'t open input file; error $!";
open(FOUT,$output_name) or die "Can\'t open output file ; error $!";
$kb=0;

while ($line = <FIN>) {
	if ($line =~ m/^\d\d\d\d\d\d /) {   # get the date and record number
		if ($line =~ m/\d 00[1234]/) {    # just select 001 - 004 records
			if ((++$kb%100000) == 0) {
				print $kb,"\n";   # show that the program is running
				if (($kb%500000) == 0) {
					close(FOUT) or die "Can\'t close output file ; error $!";
					++$kfile;
					$output_name = ">select.$kfile.".$ARGV[0];
					open(FOUT,$output_name) or die "Can\'t open output file ; error $!";
					print "New file: $output_name\n";
				}
			}
			$head = $line;
			$line = <FIN>;
			if ($line !~/^"/) {  # skip lines beginning with quotes
				print FOUT $head;
				print FOUT $line;
				while ($line = <FIN>) {
					print FOUT $line;
					if (length($line) == 1) {last;}
				}
			}
		}
	}
}
print "Finished reading...\n";
close(FOUT) or die "Can\'t close output file ; error $!";
close(FIN) or die "Can\'t close input file ; error $!";
print "Program has finished!\n";
