##  LN.data.seqsort.pl
##
##  This Perl program takes a TABARI-formatted input file with LN record headers and
##  restores it to record-sequence order following the running of LN.data.dups.pl
##   
##  TO RUN PROGRAM:
##
##  perl LN.data.seqsort.pl filename
##
##  where filename is the name of file to be sorted. Output will be in the file
##  seqsort.filename 
##
##  PROGRAMMING NOTES:
##
##  1. This seems to work only if the file format is Unix; didn't work with Windows
##     formated files.
##
##  SYSTEM REQUIREMENTS
##  This program has been successfully run under Mac OS 10.5; it is standard perl
##  so it should also run in Unix or Windows. 
##
##  PROVENANCE:
##  Programmer: Philip A. Schrodt
##              Kansas Event Data System Project
##              Center for International Political Analysis
##              University of Kansas 
##	            Lawrence, Kansas, 66045 U.S.A.
##	            http://web.ku.edu/keds
##
##	Copyright (c) 2009  Philip A. Schrodt.  All rights reserved.
##
## 	Redistribution and use in source and binary forms, with or without modification,
## 	are permitted under the terms of the GNU General Public License:
## 	http://www.opensource.org/licenses/gpl-license.html
##
##	Report bugs to: schrodt@ku.edu
##
##	For plausible indenting of this source code, set the tab size in your editor to "2"
##
##  REVISION HISTORY:
##  16-Jun-09:  Initial version
##
##  ----------------------------------------------------------------------------------

#!/usr/local/bin/perl

# ======== main program =========== #

open(FIN,$ARGV[0])  or die "Can\'t open input file; error $!";
open(FOUT,">seqsort.$ARGV[0]") or die "Can\'t open output file ; error $!";
@notsort = ();
@sorted  = ();   # probably unnecessary...

$cur = "";
$ka = 0;
while ($line = <FIN>) {
	if ($line =~ m/^(\d\d\d\d\d\d) \d\d\d/) {   # get the date and record number
		$new = $line;
		while ($line = <FIN>) {
			if (length($line) == 1) {last;}
			$new = $new.$line;
		}
		if ($1 eq $cur) {
			push @notsort, $new;
		}
		else {
			@sorted = sort {substr($a,0,20) cmp substr($b,0,20)	} @notsort; # sort on the date and sequence ID
			print $cur,"\n";
			foreach $line (@sorted) {
				print FOUT $line,"\n";
			}
			@notsort = ();
			push @notsort, $new;
			@sorted  = ();   # probably unnecessary...
			$cur = $1; # probably should be closer to the match
		}
	}
}
close(FOUT) or die "Can\'t close output file ; error $!";
close(FIN) or die "Can\'t close input file ; error $!";
print "Program has finished!\n";
